//=============================================================================
//  Unit IPL5xCom : assure la l'interface entre Delphi et IPL5Xcom.DLL
//
//  Reprend les fonctions de IPL5Xcom.dll
//  Dclare les Types, Constantes et Variables ncessaires  la com. avec ILP5X
//  Propose des fonctions supplmentaires pour les traitements usuels
//  Assure la gestion des changes avec la DLL par SAFEARRAYS et POINTEURS
//
//  (c) Jean-Luc Soumard / Thierry Leuba  2012
//
//  Version 0.1 Beta - premire diffusion le 28/10/2012
//
//  Historique des mises  jour
//  Version initiale du 06/09/2012
//  09/09/2012 20h30 : JLS : dernire modification version bta
//  13/09/2012         THL : Macros Flash
//  14/09/2012         THL : Adaptation IPL5X_LCD_Write pour 1x16 et 4x20 lignes
//  15/09/2012 21h10   JLS : Remplacement Record par Packed Record pour matriser la structure
//  16/09/2012 18h00   JLS : Macros table : GetActiveTable, SetActiveTable, ReadTable
//  17/09/2012         THL : Macros gestion IO1
//  18/09/2012         JLS : Macro table WriteTable (non test)
//  23/09/2012         JLS : Macros fins de course et palpeur (non test)
//  23/09/2012 17h30   THL : Mise  jour IPL5X_LCD_Write pour 2x20 et 4x20 lignes
//  25/09/2012         JLS : Ajout de constantes pour dcodages (Frequency, BufferUse..)
//  28/10/2012         THL : Ajout des fonctions IPL5X_LCD_SetCGRAM et IPL5X_LCD_ChrWrite
//                            Ces fonctions sont utilisable  partir de IPL5X 0.26
//
//  Cette unit est fournie en l'tat sans aucune garantie, les auteurs ne
//  peuvent pas tre tenus pour responsables d'ventuels problmes causs par
//  un mauvais fonctionnement des routines proposes.
//
//  Cette unit ne peut pas tre intgre dans un projet commercial sans
//  l'autorisation crite des auteurs.
//
//  Un grand merci  Pascal pour l'aide apport lors du dveloppement de ces applications.
//=============================================================================

                                                                               
//=============================================================================

unit IPL5xCom;

interface

uses
  Windows,Messages,Dialogs,ActiveX,sysutils;

//=============================================================================
type  
//=============================================================================
  BufferIPL5X   = Array[0..36] of byte;   // structure de base pour la communication
  pBufferIPL5X  = ^BufferIPL5X;           // pointeur vers une structure de base
  PPSafeArray   = ^PSafeArray;            // pointeur de pointeur de SAFEARRAY (PSafeArray dclar dans ActiveX)
  PLongInt      = ^LongInt;               // pointeur vers un entier long

  // Structures "volues" pour la manipulation des donnes entre PC et IPL5X et retour

  // Entier sur 24 bits
  IPL5X_SEG = record                      // Numro de segment actif sur 24 bits
    L     : Byte;                         // SEG = L + 256*M + 65536*H
    M     : Byte;
    H     : Byte;
  end;

  // pour envoi de DATAS depuis le PC->IPL5X
  Data_PC_IPL5X = packed record           // Donnes type DATA envoye du PC vers IPL5X
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$44 pour DATA
    CMD   : Byte;                         // Octet de commande avec CMD bit5=1 et bit6=1 pour donnes compltes
    NBR   : LongWord;                     // Nombre de pulses
    S     : Array[1..5] of LongWord;      // Nombre de pas par axe
    PWM   : Byte;                         // PWM
    F_ACC : Byte;                         // Index de frquence d'acclration
    F_DCC : Byte;                         // Index de frquence de dclration
    DEC   : LongWord;                     // Nombre de pulses avant dclration
  end;
  PData_PC_IPL5X = ^Data_PC_IPL5X;        // Pointeur vers la structure Data

  // pour exploitation du retour IPL5X->PC suite  DATA ou INFORMATIONS
  Info_IPL5X_PC = packed record           // Informations en retour d'IPL5X (retour sur INFO ou DATA)
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$44 pour retour DATA ; $49 pour retour INFORMATIONS
    ST    : Byte;                         // $00 = buffer pas plein ; $FF = buffer plein
    BUF   : Byte;                         // niveau de remplissage du buffer
    SEG   : IPL5X_SEG;                    // numro du segment en cours
    FLAG  : Byte;                         // dtails sur notice dveloppeur IPL5X
    INPUT : Byte;                         // dtails sur notice dveloppeur IPL5X (tat des switchs d'entr)
    PWMA  : Byte;                         // valeur PWM en mode auto
    PWMM  : Byte;                         // valeur PWM en mode manuel
    KEYB  : Byte;                         // retour tat du clavier, dtails sur notice dveloppeur IPL5X
  end;
  PInfo_IPL5X_PC = ^Info_IPL5X_PC;        // Pointeur vers la structure Info

  // pour exploitation du retour IPL5X->PC suite  STOP
  Stop_IPL5X_PC = packed record           // permet l'exploitation des infos en retour de STOP (avec arrt ou non)
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$53 pour retour STOP
    RET   : Byte;                         // indicateur d'activit avant et suite au STOP
    SEG   : IPL5X_SEG;                    // numro du segment en cours lors du stop
    NBR   : LongWord;                     // nombre de pulses restant sur SEG aprs le STOP
  end;
  PStop_IPL5X_PC = ^Stop_IPL5X_PC;        // Pointeur vers la structure Stop

  // pour exploitation des tats du clavier et des entres
  IPL5X_Input = packed record
    KeyUp      : Boolean;                 // Bouton Flche haut
    KeyDown    : Boolean;                 // Bouton Flche bas
    KeyRight   : Boolean;                 // Bouton Flche droite
    KeyLeft    : Boolean;                 // Bouton Flche gauche
    KeyOK      : Boolean;                 // Bouton OK
    KeyESC     : Boolean;                 // Bouton ESC
    KeySTOP    : Boolean;                 // Bouton STOP PROG
    KeyManAuto : Boolean;                 // Interrupteur PWM Auto / Manuel
    KeyOnOff   : Boolean;                 // Interrupteur PWM On / Off
    Sensor     : Boolean;                 // Capteur (outil, pice etc...)
    Limit      : Boolean;                 // Contacts fin de course
    PWM        : Byte;                    // % au potentiomtre PWM
  end;

  // pour traitement de l'instruction TABLE de IPL5X
  IPL5X_Table = packed record             // Reprend toutes les caractristiques d'une table
    NBR   : Byte;                         // Numro de table entre 0, 1 ou 2
    NAME  : Array[1..8] of Char;          // Nom de la table
    A     : Array[1..5] of Byte;          // Nom de l'axe ( dcoder)
    OUT   : Array[1..10] of Byte;         // Fonction des 10 sorties
    S     : Array[1..5] of Word;          // Nb de pas pour 1mm ou 1 par axe
    VMSA  : Array[1..5] of Word;          // Vitesse maxi sans acc en pas/s par axe
    VMAA  : Array[1..5] of Word;          // Vitesse maxi avec acc en pas/s par axe
    ORI   : Array[1..5] of Word;          // Offset d'origine pour chaque axe en nb de pas
    FREQ  : Byte;                         // Frquence pulses : 1=10kHz, 2=20kHz, 3=30KHz, 4=40kHz, 5=50kHz
    SLOPE : Byte;                         // Rampe d'acc maxi : entre $00 et $0F
    PWM   : Byte;                         // PWM maxi entre $00 et $FF (0  100%)
    MOFF  : Byte;                         // Nb secondes entre dernier pas et arrt moteurs (0  127)
    FLAGS : Byte;                         // pour dtail voir doc IPL5X
    IO1   : Byte;                         // paramtrage IO1, dtail dans la doc
    LANG  : Byte;                         // Langue : 0=franais , 1=anglais
  end;

  IPL5X_Table_Low = packed record         // Pour changes avec IPL5X Table Low
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$54 pour TABLE
    CMD   : Byte;                         // =$03 pour READ LOW ; $04 pour WRITE LOW
    NBR   : Byte;                         // Numro de table entre 0, 1 ou 2
    NAME  : Array[1..8] of Char;          // Nom de la table
    A     : Array[1..5] of Byte;          // Nom de l'axe ( dcoder)
    OUT   : Array[1..10] of Byte;         // Fonction des 10 sorties
    FREQ  : Byte;                         // Frquence pulses : 1=10kHz, 2=20kHz, 3=30KHz, 4=40kHz, 5=50kHz
    SLOPE : Byte;                         // Rampe d'acc maxi : entre $00 et $0F
    PWM   : Byte;                         // PWM maxi entre $00 et $FF (0  100%)
    MOFF  : Byte;                         // Nb secondes entre dernier pas et arrt moteurs (0  127)
    FLAGS : Byte;                         // pour dtail voir doc IPL5X
    IO1   : Byte;                         // paramtrage IO1, dtail dans la doc
    LANG  : Byte;                         // Langue : 0=franais , 1=anglais
  end;
  PIPL5X_Table_Low = ^IPL5X_Table_Low;    // Pointeur vers la structure Table Low

  IPL5X_Table_High = packed record        // Pour changes avec IPL5X Table High
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$54 pour TABLE
    CMD   : Byte;                         // =$05 pour READ HIGH ; $06 pour WRITE HIGH
    NBR   : Byte;                         // Numro de table entre 0, 1 ou 2
    S     : Array[1..5] of Word;          // Nb de pas pour 1mm ou 1 par axe
    VMSA  : Array[1..5] of Word;          // Vitesse maxi sans acc en pas/s par axe
    VMAA  : Array[1..5] of Word;          // Vitesse maxi avec acc en pas/s par axe
  end;
  PIPL5X_Table_High = ^IPL5X_Table_High;  // Pointeur vers la structure Table High

  IPL5X_Table_Upper = packed record       // Pour changes avec IPL5X Table Upper
    ID    : Byte;                         // ID Record = 0
    INST  : Byte;                         // =$54 pour TABLE
    CMD   : Byte;                         // =$07 pour READ UPPER ; $08 pour WRITE UPPER
    NBR   : Byte;                         // Numro de table entre 0, 1 ou 2
    ORI   : Array[1..5] of Word;          // Offset d'origine pour chaque axe en nb de pas
  end;
  PIPL5X_Table_Upper = ^IPL5X_Table_Upper;// Pointeur vers la structure Table Upper


//*****************************
  // pour exploitation du retour IPL5X->PC suite  Info_IPL_Flash
  Info_Flash_IPL5X_PC = packed record     // permet l'exploitation des infos en retour
    ID    : Byte;                         // ID Record = 0
    Code  : Byte;                         // Code retour de fonction
    Fab   : Byte;			              // Etat de protection d'criture
    Flash : Byte;                         // Type de Flash
    Size  : Byte;                         // Taille en Mb
    WPS   : Byte;			              // Etat de protection d'criture
  end;
  PInfo_Flash_IPL5_PC = ^Info_Flash_IPL5X_PC;// Pointeur vers la structure InfoFlash

  // pour exploitation du retour IPL5X->PC suite  IPL5X_Set_IO1
  IO1_IPL5X_PC = packed record            // permet l'exploitation des infos en retour
    ID         : Byte;                    // ID Record = 0
    Code       : Byte;                    // Code retour de fonction
    CMD        : Byte;                    // Etat en Byte, 1 = actif
    VAL        : Byte;                    // Setting en Byte,  dcoder
    //IsON       : Boolean;               // 1 = actif
    //IsInput    : Boolean;               // 1 = entre
    //DefState   : Boolean;			      // Etat sortie par dfaut
    //FollowPWM  : Boolean;               // 1 = La sortie suit PWM
  end;
  PIO1_IPL5X_PC = ^IO1_IPL5X_PC;          // Pointeur vers la structure Set_IO1

//=============================================================================
const 
//=============================================================================
  MaxBuffer             = 36;

  // Codes de la puce...
  IPL_VENDOR_ID         = $04D8;
  IPL_PRODUCT_ID        = $00AA;

  // Codes fonctions IPL5x...
  IPL_VERSION           = $56;
  IPL_BUFFER            = $42;
  IPL_DATA              = $44;
  IPL_INFO              = $49;
  IPL_STOP              = $53;
  IPL_TABLE             = $54;
  IPL_PWM               = $50;
  IPL_MOTORS            = $4D;
  IPL_IO1               = $47;
  IPL_OVERRIDE          = $4F;
  IPL_LCD               = $4C;
  IPL_RESET             = $5A;

  IPL_FLASH_MODEL       = $46;
  IPL_FLASH_READ        = $52;
  IPL_FLASH_BLOCKERASE  = $45;
  IPL_FLASH_WRITE       = $57;
  IPL_FLASH_CONTENT     = $42;

  // Paramtres et codes usuels
  MT_ON                 = $01;      // CMD Motors ==> ON
  MT_OFF                = $00;      // CMD Motors ==> OFF
  MT_CURSTAT            = $02;      // CMD Motors ==> Lire le "current status"
  MT_SETAUTO            = $03;      // CMD Motors ==> Fixe le temps avant arrt des moteurs
  STP_FAST              = $00;      // CMD Stop   ==> arrt instantan
  STP_SLOW              = $01;      // CMD Stop   ==> arrt avec dclration (ncessite paramtre FDEC)
  STP_STATUS            = $03;      // CMD Stop   ==> pour retour de status = GetStatus
  BUF_RESET             = $00;      // CMB Buffer ==> reset buffer et stop les dplacements
  BUF_USB               = $01;      // CMB Buffer ==> source USB
  BUF_FLASH             = $02;      // CMB Buffer ==> source FLASH ==> ncessite BCK et ADR
  BUF_RUN               = $80;      // CMB Buffer ==> lance l'excution du contenu du buffer
  DATA_END              = $80;      // CMD Data   ==> END DATA, fin de l'envoi de dplacements
  RST_NORMAL            = $00;      // BLDR Reset ==> Reboot en mode normal
  RST_BOOTLOADER        = $01;      // BLDR Reset ==> Reboot en mode bootloader
  LCD_CLEAR             = $05;      // CMD LCD    ==> Clear Display
  LCD_CGRAM             = $06;      // CMD LCD    ==> Eciture en CGRAM

  // Dcodage clavier et contacts masques a utiliser pour tests AND ou OR
  KEY_ESC               = $01;      // masque keyb 1
  KEY_OK                = $02;      // masque keyb 2
  KEY_LEFT              = $10;      // masque keyb 10
  KEY_DOWN              = $20;      // masque keyb 32
  KEY_UP                = $40;      // masque keyb 64
  KEY_RIGHT             = $80;      // masque keyb 128
  KEY_AUTOMAN           = $01;      // masque input 1  -> 1 = man
  KEY_ONOFF             = $08;      // masque input 8  -> 1 = Off
  KEY_STOP              = $20;      // masque input 32 -> 0 = Stop invers !!
  IMP_AUTOMAN           = $01;      // bit 0 de INPUT  -> 1 = man
  IMP_LIMIT             = $02;      // bit 1 de INPUT  -> 0 = ferm , 1 = ouvert
  IMP_SENSOR            = $04;      // bit 2 de INPUT  -> 0 = ferm , 1 = ouvert
  IMP_ONOFF             = $08;      // bit 3 de INPUT  -> 1 = Off
  IMP_IO1               = $10;      // bit 4 de INPUT  -> 0 = GND , 1 = +5V
  IMP_STOP              = $20;      // bit 5 de INPUT  -> 0 = Stop invers !!

  // Pour grer IO1
  IO1_OFF                = $00;      // Mise  0 de IO1
  IO1_ON                 = $01;      // Mise  1 de IO1
  IO1_READ               = $02;      // Lecture tat de IO1
  IO1_SETTING            = $03;      // Paramtrage de la PIN
  IO1_IsON               = true;     // Etat entre active
  IO1_IsOFF              = false;    // Etat entre inactive
  IO1_IsOUTPUT           = true;     // IO1 est une sortie, false c'est une entre
  IO1_IsPWM              = true;     // IO1 actif avec PWM, false commande libre

  // Pour Palpeur et Fin de course
  OVR_ON                 = $00;      // Active les fins de course et palpeurs
  OVR_OFF                = $01;      // Dsactive la prise en compte des fins de course et palpeur
  OVR_STATUS             = $02;      // Lecture de l'tat des fins de course
  OVR_USE                = $01;      // Masque utilisation 0=prise en compte ; 1=ignorer
  OVR_TABLE              = $02;      // Masque table configure 0=sans ; 1=avec
  OVR_AXIS_LIMIT         = $04;      // Masque fin de course : 0=ferm ; 1=ouvert
  OVR_SENSOR             = $08;      // Masque palpeur : 0=ferm ; 1=ouvert

  // Pour grer les messages windows USB si ncessaire
  WM_DEVICECHANGE       = $0219;
  DBT_DEVNODES_CHANGED  = $0007;
  WM_HID_EVENT          = WM_APP+200;
  NOTIFY_PLUGGED        = $0001;
  NOTIFY_UNPLUGGED      = $0002;
  NOTIFY_CHANGED        = $0003;
  NOTIFY_READ           = $0004;

  // Dcodage de la puce Flash
  AmicType   : array [0..8] of string = ('000','512','010','020','040','080','016','032','A25L');
  AmicMb     : array [0..7] of string = ('0','0.5','1','2','4','8','16','32');

  // Dcodage du nom des axes (paramtre A des donnes TABLE) les '-' correspondent aux codes non utiliss
  //   Nom de l'axe i = AxisName[Table.A[i]]
  AxisName   : array [0..63] of string =
                ( '-', '-', '-', '-', 'X','XL','XR', '-', 'Y','YL','YR', '-', 'Z','ZL','ZR', '-',  // 00  0F
                  'A', 'B', 'C', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-',  // 10  1F
                  'R','RX','RY','RZ', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-',  // 20  2F
                  'U', 'V', 'W', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-'); // 30  3F
 //               00   01   02   03   04   05   06   07   08   09   0A   0B   0C   0D   0E   0F

  // Frquences de base des pulses en Hz : index=Table.FREQ
  Frequency  : array [1..5] of LongInt = (10000,20000,30000,40000,50000);

  // Taux de remplissage du buffer de datas en pourcentage : = retour BUF de INFORMATIONS
  BufferUse  : array [0..7] of LongInt = (0,14,29,43,57,71,86,100);

  // Nom de la DLL...
  Nom_DLL = 'IPL5xCom.dll';
  VUnitIPL5xCom = 'V 0.1 beta';

//=============================================================================
var  
//=============================================================================
  saData            : PSafeArray;        // Pointeur sur SAFEARRAY pour communication avec la DLL
  DataArrayBounds   : TSafeArrayBound;   // Structure du SAFEARRAY saData (initialis avec IPL5X_OpenCom
  ComOpen           : Boolean;           // Vrai si SAFEARRAY cr (IPL5X_OpenCom a t excut)

  // Fonctions de la DLL...
  function IPL5X_Dll_Version :Longint; stdcall; external Nom_DLL name 'IPL5X_Dll_Version';
  function IPL5X_IsConnected :Longint; stdcall; external Nom_DLL name 'IPL5X_IsConnected';
  function IPL5X_Open :Longint; stdcall; external Nom_DLL name 'IPL5X_Open';
  function IPL5X_Close : bool ; stdcall; external Nom_DLL name 'IPL5X_Close';
  function IPL5X_Send( Data:PPSafeArray ; Fast:PLongInt):Longint; stdcall; external Nom_DLL name 'IPL5X_Send';

  // Fonctions de communication avec pr et post traitement (assurent la gestion des SAFEARRAY)
  function IPL5X_Error(Err:integer):String;
  function IPL5X_OpenCom : Longint ;
  function IPL5X_CloseCom : Longint ;
  function IPL5X_Version : String;
  procedure IPL5X_Reset( BLDR : Byte=$00) ;
  function IPL5X_Command(var Envoi : BufferIPL5X ; var Retour : BufferIPL5X) : Longint ;
  function IPL5X_Data(var Envoi : Data_PC_IPL5X ; var Retour : Info_IPL5X_PC): Longint ;
  function IPL5X_Information(var Retour : Info_IPL5X_PC): Longint ;
  function IPL5X_GetInput(var Input:IPL5X_Input) : Longint;
  function IPL5X_Stop(var CMD:Byte ; FDEC:Byte ; var Retour : Stop_IPL5X_PC): Longint ;
  function IPL5X_GetStatus(var Retour : Stop_IPL5X_PC) : Longint;
  function IPL5X_Buffer(var CMD:Byte ; BCK:Byte=0 ; ADR:Word=0 ) : Longint;
  function IPL5X_Motors(var CMD:Byte ; MOFF:Byte) :Longint;
  function IPL5X_MotOnOff(CMD:Boolean) :Longint;
  function IPL5X_LCD_Clear : Longint;
  function IPL5X_LCD_Write(LINE : Byte ; POS : Byte ; TEXT : String) : Longint;
  function IPL5X_LCD_SetCGRAM(CHAR,C0,C1,C2,C3,C4,C5,C6,C7 : Byte ) : Longint;
  function IPL5X_LCD_ChrWrite(LINE,POS,CHAR : Byte ) : Longint;
  function IPL5X_GetActiveTable(var NBR : Byte) : Longint;
  function IPL5X_SetActiveTable(var NBR : Byte ; var RET : Byte) : Longint;
  function IPL5X_ReadTable(var NBR : Byte ; var Table : IPL5X_Table) : Longint;
  function IPL5X_WriteTable(NBR : Byte ; Table : IPL5X_Table) : Longint;
  function IPL5X_StatusOverride(var OVER:boolean; var SENSOR:boolean; var USE:boolean) : Longint;
  function IPL5X_UseOverride(USE:boolean) : Longint;

//********************** CHANTIER THL

 //Fonctions de gestion d'IO1
  function IPL5X_Cde_IO1(var CMD:Byte; VAL:Byte; Retour : IO1_IPL5X_PC) :Longint;
 //Fonctions d'utilisation de la Flash externe
  function IPL5X_InfoFlash(var Retour : Info_Flash_IPL5X_PC): Longint ;
 //function IPL5x_FlashType        ($46) Test type de flash prsente
 //function IPL5x_FlashRd          ($52, BCK,ADRH,ADRL)Lecture d'un block en flash
 //function IPL5x_FlashBlkEraseCLR ($45, BCK)  Effacement d'un block en flash
 //function IPL5x_FlashWr          ($57, BCK,ADRH,ADRL,...)Ecriture d'un block en flash

//**********************

//=============================================================================
implementation 
//=============================================================================


//-----------------------------------------------------------------------------
function IPL5X_Error(Err:integer):String;
//-----------------------------------------------------------------------------
// Appel  : Numro de l'erreur
// Retour : Texte en clair du code d'erreur IPL5Xcom
//-----------------------------------------------------------------------------

begin
  case Err of
       1 : IPL5X_Error:= '(1) Envoi/rception OK, trame valide' ;
    -101 : IPL5X_Error:= '(-101) Table introuvable' ;
    -102 : IPL5X_Error:= '(-102) Table trop petite' ;
    -200 : IPL5X_Error:= '(-200) IPL5x introuvable' ;
    -201 : IPL5X_Error:= '(-201) Communication impossible' ;
    -202 : IPL5X_Error:= '(-202) Erreur PC -> IPL5x' ;
    -203 : IPL5X_Error:= '(-203) Erreur IPL5x -> PC' ;
    -204 : IPL5X_Error:= '(-204) Erreur Data IPL5x <-> PC' ;
   -1000 : IPL5X_Error:= '(-1000) Err IPL5X_OpenCom';
   -1001 : IPL5X_Error:= '(-1001) IPL5X_OpenCom pas excute';
   -1100 : IPL5X_Error:= '(-1100) Err IPL5X_CloseCom';
   -1200 : IPL5X_Error:= '(-1200) Err IPL5X_Command';
   -1300 : IPL5X_Error:= '(-1300) Err IPL5X_Data';
   -1400 : IPL5X_Error:= '(-1400) Err IPL5X_Information';
   -1410 : IPL5X_Error:= '(-1410) Err IPL5X_GetInput';
   -1500 : IPL5X_Error:= '(-1500) Err IPL5X_Stop';
   -1600 : IPL5X_Error:= '(-1600) Err IPL5X_GetStatus';
   -1700 : IPL5X_Error:= '(-1700) Err IPL5X_Buffer';
   -1800 : IPL5X_Error:= '(-1800) Err IPL5X_Motors';
   -1810 : IPL5X_Error:= '(-1810) Err IPL5X_MotOnOff';
   -1900 : IPL5X_Error:= '(-1900) Err IPL5X_LCD_Clear';
   -1910 : IPL5X_Error:= '(-1910) Err IPL5X_LCD_Write';
   -1920 : IPL5X_Error:= '(-1920) Err IPL5X_LCD_SetCGRAM';
   -1930 : IPL5X_Error:= '(-1930) Err IPL5X_LCD_ChrWrite';
   -2000 : IPL5X_Error:= '(-2000) Err IPL5X_InfoFlash';
   -2100 : IPL5X_Error:= '(-2100) Err IPL5X_GetActiveTable';
   -2110 : IPL5X_Error:= '(-2110) Err IPL5X_SetActiveTable';
   -2120 : IPL5X_Error:= '(-2120) Err IPL5X_ReadTable';
   -2130 : IPL5X_Error:= '(-2130) Err IPL5X_WriteTable';
   -2200 : IPL5X_Error:= '(-2200) Err IPL5X_StatusOverride';
   -2250 : IPL5X_Error:= '(-2250) Err IPL5X_UseOverride';
  else
    IPL5X_Error:= '(9999) Code inconnu' ;
  end ;
end;

//-----------------------------------------------------------------------------
function IPL5X_OpenCom; // : Longint ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Code d'erreur
//-----------------------------------------------------------------------------
// Description :
// Ouvre la communication avec IPL5X et cration du SAFEARRAY ncessaire aux changes.
// Le PSAFEARRAY saData est cr comme une table de 0  36 bytes, le bit 0 sera toujours mis  0
// Intgre la gestion des SAFEARRAY et des POINTEURS
// ATTENTION CETTE FONCTION DOIT IMPERATIVEMENT ETRE APPELEE EN DEBUT DE PROGRAMME
//-----------------------------------------------------------------------------

var
  Retour : Longint;
begin
  try
    ComOpen:=False;
    Retour:=IPL5X_Open;
    if Retour=1 then
    begin
      DataArrayBounds.lLbound:= 0;                            // premier lment  l'index 0
      DataArrayBounds.cElements:= 37;                         // 37 lments
      saData := SafeArrayCreate( VT_I1, 1, DataArrayBounds ); // cration de Data de Byte (=VT_I1)
      ComOpen:=True;
    end;
    IPL5X_OpenCom:=Retour;
  except
    ComOpen:=False;
    IPL5X_OpenCom:=-1000;                             // erreur lors de l'excution de IPL5X_OpenCom
    ShowMessage('Except dans IPL5X_OpenCom.');        // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_CloseCom; // : Longint ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Code d'erreur
//-----------------------------------------------------------------------------
// Description :
// Ferme la communication avec IPL5X et dtruit le SAFEARRAY pour librer la mmoire
// Intgre la gestion des SAFEARRAY et des POINTEURS
//------------------------------------------------------------------------------

begin
  IPL5X_CloseCom:=-1100;                              // erreur lors de l'excution de IPL5X_CloseCom
  try
    if ComOpen then
    begin
      SafeArrayDestroy(saData);                       // pour librer l'emplacement mmoire
    end;
    if IPL5X_Close then IPL5X_CloseCom:=1;
    ComOpen:=False;
  except
    ComOpen:=False;
    ShowMessage('Except dans IPL5X_CloseCom.');       // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Version; // : String ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Texte de version ou d'erreur
//-----------------------------------------------------------------------------
// Description :
// Interroge IPL5X et retourne la version du firmware
// Retourne un texte de version ou d'erreur si problme
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
  Err     : Longint;
  i       : Integer;
  st      : string;
begin
  try
    if not ComOpen then begin IPL5X_Version:='OpenCom'; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0]      := 0;                               // le premier lment doit toujours tre  0, donc au cas o...
    Envoi[1]      := IPL_VERSION;                     // Instruction Version
    pBuff         := saData.pvData;                   // pBuff pointe sur les datas du safearray
    pBuff^        := Envoi;                           // on met le contenu de Envoi  l'adresse du safearray
    Fast          := 0;                               // 0 pour avoir les tests, 1 pour avoir la rapidit
    Err           := IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    Envoi         := pBuff^;                          // Envoi prend le contenu de l'adresse des donnes SAFEARRAY
    st:='';
    for i:=2 to 13 do st:=st+Chr(Envoi[i]);
    IPL5X_Version := st;
  except
    IPL5X_Version := 'Erreur';                        // erreur lors de l'excution de IPL5X_Command
    ShowMessage('Except dans IPL5X_Version.');        // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
procedure IPL5X_Reset; // (BLDR : Byte=RST_NORMAL);
//-----------------------------------------------------------------------------
// Appel  : Variable RST_NORMAL (dfaut) ou RST_BOOTLOADER (=$01)
// Retour : Nant
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande RESET  IPL5X
// Par dfaut reboot normal d'IPL5X, RST_BOOTLOADER (=$01) autorise le flash firmware
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
  Err     : Longint;
begin
  try
    if not ComOpen then exit;                         // IPL5X_OpenCom pas lanc
    Envoi[0]      := 0;                               // le premier lment doit toujours tre  0, donc au cas o...
    Envoi[1]      := IPL_RESET;                       // Instruction Reset
    Envoi[2]      := $55;
    Envoi[3]      := $AA;
    Envoi[4]      := BLDR;                            // par dfaut $00
    pBuff         := saData.pvData;                   // pBuff pointe sur les datas du safearray
    pBuff^        := Envoi;                           // on met le contenu de Envoi  l'adresse du safearray
    Fast          := 0;                               // 0 pour avoir les tests, 1 pour avoir la rapidit
    Err           := IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    ShowMessage('Except dans IPL5X_Reset.');        // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Command;  // (var Envoi : BufferIPL5X ; Retour : BufferIPL5X) : Longint ;
//-----------------------------------------------------------------------------
// Appel  : Contenu de BufferIPL5X tableau en bytes
// Retour : Contenu de BufferIPL5X tableau en bytes, Code d'erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi d'un tableau de BYTE et le fait voluer avec les infos renvoyes par IPL5X
// Fonction IPL5X_Send de la DLL avec le traitement des SAFEARRAY intgr
// Permet de traiter n'importe quel change PC / IPL5X
//-----------------------------------------------------------------------------

var
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_Command:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0]      :=0;                                // le premier lment doit toujours tre  0, donc au cas o...
    pBuff         := saData.pvData;                   // pBuff pointe sur les datas du safearray
    pBuff^        := Envoi;                           // on met le contenu de Envoi  l'adresse du safearray
    Fast          := 1;                               // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Command := IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    Retour        := pBuff^;                          // Retour prend le contenu de l'adresse des donnes SAFEARRAY
  except
    IPL5X_Command := -1200;                           // erreur lors de l'excution de IPL5X_Command
    ShowMessage('Except dans IPL5X_Command.');        // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Data;  // (var Envoi : Data_PC_IPL5X ; Retour : Info_IPL5X_PC): Longint ;
//-----------------------------------------------------------------------------
// Appel  : Contenu de structure Data_PC_IPL5X
// Retour : Contenu de structure Info_IPL5X_PC, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoie la structure DATA vers IPL5X et traite le retour sous forme de structure INFORMATIONS
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  pData   : PData_PC_IPL5X;
  pInfo   : PInfo_IPL5X_PC;
  Fast    : Longint;
begin
  try
    if not ComOpen then begin IPL5X_Data:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi.ID   := 0;                                  // le premier lment doit toujours tre  0, donc au cas o...
    Envoi.INST := IPL_DATA;                           // force INST  $44
    pData      := saData.pvData;                      // pData pointe sur les datas du safearray
    pData^     := Envoi;                              // on met le contenu de Envoi  l'adresse du safearray
    Fast       := 1;                                  // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Data := IPL5X_Send(@saData,@Fast);          // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pInfo      := saData.pvData;                      // pInfo pointe sur les datas du safearray
    Retour     := pInfo^;
  except
    IPL5X_Data:=-1300;                                // erreur lors de l'excution de IPL5X_Data
    ShowMessage('Except dans IPL5X_Data.');           // debug uniquement
  end;
end;


//-----------------------------------------------------------------------------
function IPL5X_Information; // (var Retour : Info_IPL5X_PC): Longint ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Contenu de structure Info_IPL5X_PC, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoie la commande INFORMATION ($49) vers IPL5X et traite le retour sous forme de structure INFORMATION
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  pInfo   : PInfo_IPL5X_PC;
  Fast    : Longint;
begin
  try
    if not ComOpen then begin IPL5X_Information:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Retour.ID   := 0;                                 // toujours  0
    Retour.INST := IPL_INFO;                          // la commande pour "informations"
    pInfo       := saData.pvData;                     // pInfo pointe sur les datas du safearray
    pInfo^      := Retour;                            // on lui affecte les donnes contenues dans Retour
    Fast        := 1;                                 // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Information:=IPL5X_Send(@saData,@Fast);     // envoi pointeurs  IPL5X par la DLL et retour code erreur
    Retour      := pInfo^;                            // on rcupre les infos en retour
  except
    IPL5X_Information:=-1400;                         // erreur lors de l'excution de IPL5X_Information
    ShowMessage('Except dans IPL5X_Information.');    // debug uniquement
  end;
end;


//-----------------------------------------------------------------------------
function IPL5X_GetInput; //(var Input:IPL5X_Input) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Contenu de structure IPL5X_Input, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Interroge IPL5X et retourne l'tat du clavier, potentiomtre, contacts
// Envoi de la commande INFORMATION ($49) vers IPL5X et traite le retour sous forme de structure INFORMATION
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Info : Info_IPL5X_PC;
  Err  : Longint;
begin
  try
    Err := IPL5X_Information(Info);                                  // INFORMATION de IPL5X
    IPL5X_GetInput := Err;
    if Err=1 then
    begin
      Input.KeyUp      := (Info.KEYB and KEY_UP)=KEY_UP;             // Fleche haut
      Input.KeyDown    := (Info.KEYB and KEY_DOWN)=KEY_DOWN;         // Fleche bas
      Input.KeyRight   := (Info.KEYB and KEY_RIGHT)=KEY_RIGHT;       // Fleche droite
      Input.KeyLeft    := (Info.KEYB and KEY_LEFT)=KEY_LEFT;         // Fleche gauche
      Input.KeyOK      := (Info.KEYB and KEY_OK)=KEY_OK;             // Touche OK
      Input.KeyESC     := (Info.KEYB and KEY_ESC)=KEY_ESC;           // Touche ESC
      Input.KeySTOP    := not((Info.INPUT and IMP_STOP)=IMP_STOP);   // Touche STOP
      Input.KeyManAuto := (Info.INPUT and IMP_AUTOMAN)=IMP_AUTOMAN;  // inter PWM Manuel/Auto
      Input.KeyOnOff   := (Info.INPUT and IMP_ONOFF)=IMP_ONOFF;      // inter on/off
      Input.Sensor     := (Info.INPUT and IMP_SENSOR)=IMP_SENSOR;    // palpeurs
      Input.Limit      := (Info.INPUT and IMP_LIMIT)=IMP_LIMIT;      // fins de course
      Input.PWM        := round(100*Info.PWMM/255);                  // potentiomtre
    end
    else
    begin
      IPL5X_GetInput := Err;                          // pour relayer l'erreur
    end;
  except
    IPL5X_GetInput:=-1410;                            // erreur lors de l'excution de IPL5X_GetInput
    ShowMessage('Except dans IPL5X_GetInput.');       // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Stop; // (var CMD:Byte ; FDEC:Byte ; Retour : Stop_IPL5X_PC): Longint ;
//-----------------------------------------------------------------------------
// Appel  : CMD , FDEC
// Retour : Contenu de structure Stop_IPL5X_Input, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande STOP  IPL5X et traite le retour sous forme de structure Stop_IPL5X_PC
// Les codes ncessaires  la gestion du STOP doivent tre envoys par CMD et FDEC
// Pour la lecture de l'tat d'IPL5X, prfrer l'utilisation de IPL5X_GetStatus plutt que IPL5X_Stop
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  pStop   : PStop_IPL5X_PC;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_Stop:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0]   := 0;                                  // toujours  0
    Envoi[1]   := IPL_STOP;                           // la commande pour "stop"
    Envoi[2]   := CMD;                                // la commande du stop
    Envoi[3]   := FDEC;                               // la dclration
    pBuff      := saData.pvData;                      // pBuff pointe sur les datas du safearray
    pBuff^     := Envoi;                              // on met le contenu de Envoi  l'adresse du safearray
    Fast       := 1;                                  // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Stop := IPL5X_Send(@saData,@Fast);          // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pStop      := saData.pvData;                      // pStop pointe sur les datas du safearray
    Retour     := pStop^;                             // Retour rcupre les donnes  l'adresse du pointeur
  except
    IPL5X_Stop:=-1500;                                // erreur lors de l'excution de IPL5X_Stop
    ShowMessage('Except dans IPL5X_Stop.');           // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_GetStatus; // (var Retour : Stop_IPL5X_PC): Longint ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Contenu de structure Stop_IPL5X_PC, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande STOP/GETSTATUS  IPL5X et traite le retour sous forme de structure Stop_IPL5X_PC
// Les bons codes sont affects avant envoi  IPL5X
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  pStop   : PStop_IPL5X_PC;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_GetStatus:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_STOP;                             // la commande pour "stop"
    Envoi[2] := STP_STATUS;                           // la commande du stop pour lire l'tat actuel sans stopper
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_GetStatus:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pStop    := saData.pvData;                        // pStop pointe sur les datas du safearray
    Retour   := pStop^;                               // Retour rcupre les donnes  l'adresse du pointeur
  except
    IPL5X_GetStatus:=-1600;                           // erreur lors de l'excution de IPL5X_GetStatus
    ShowMessage('Except dans IPL5X_GetStatus.');      // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Buffer;  // (var CMD:Byte ; BCK:Byte=0 ; ADR:Word=0 ) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande, Bloc, Adresse
// Retour : Commande (CMD), Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande BUFFER  IPL5X et traite le retour dans CMD
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_Buffer:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_BUFFER;                           // l'instruction BUFFER
    Envoi[2] := CMD;                                  // la commande CMD de BUFFER
    Envoi[3] := BCK;                                  // n de bloc si Flash
    Envoi[4] := ADR;                                  // ADR si Flash
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Buffer:=IPL5X_Send(@saData,@Fast);          // envoi pointeurs  IPL5X par la DLL et retour code erreur
    CMD      := pBuff^[2];                            // CMD renvoie le contenu du 3me octet des donnes SAFEARRAY
  except
    IPL5X_Buffer:=-1700;                              // erreur lors de l'excution de IPL5X_Buffer
    ShowMessage('Except dans IPL5X_Buffer.');         // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_Motors; //(var CMD:Byte ; MOFF:Byte) :Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande, Etat moteur
// Retour : Commande (CMD), Etat moteur (MOFF),Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande MOTORS  IPL5X et traite le retour dans CMD et MOFF
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_Motors:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_MOTORS;                           // l'instruction MOTORS
    Envoi[2] := CMD;                                  // la commande CMD de MOTORS
    Envoi[3] := MOFF;                                 // MOFF temps avant arrt des moteurs
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Motors:=IPL5X_Send(@saData,@Fast);          // envoi pointeurs  IPL5X par la DLL et retour code erreur
    CMD      := pBuff^[2];                            // CMD renvoie le contenu du 3me octet des donnes SAFEARRAY
    MOFF     := pBuff^[3];                            // MOFF renvoie le contenu du 4me octet des donnes SAFEARRAY
  except
    IPL5X_Motors:=-1800;                              // erreur lors de l'excution de IPL5X_Motors
    ShowMessage('Except dans IPL5X_Motors.');         // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_MotOnOff; //(CMD:Boolean) :Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande moteur
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande MOTORS  IPL5X paramtre pour ON ou OFF des moteurs
// CMD=True pour marche, CMD=False pour arrt des moteurs
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_MotOnOff:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_MOTORS;                           // l'instruction MOTORS
    if CMD then Envoi[2]:=MT_ON else Envoi[2]:=MT_OFF;  // la commande ON ou OFF des moteurs
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_MotOnOff:=IPL5X_Send(@saData,@Fast);        // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_MotOnOff:=-1810;                            // erreur lors de l'excution de IPL5X_MotOnOff
    ShowMessage('Except dans IPL5X_MotOnOff.');       // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_LCD_Clear; // : Longint;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Efface l'cran LCD de IPL5X
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_LCD_Clear:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_LCD;                              // l'instruction LCD
    Envoi[2] := LCD_CLEAR;                            // effacer LCD
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_LCD_Clear:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_LCD_Clear:=-1900;                           // erreur lors de l'excution de IPL5X_LCD_Clear
    ShowMessage('Except dans IPL5X_LCD_Clear.');      // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_LCD_Write; //(LINE : Byte ; POS : Byte ; TEXT : String) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Ligne, Position, Texte
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Accepte les afficheurs de diverses taille 1x16, 2x24,-> 4x20 caractres
// Numros de ligne valides = 1,2,3,4,la longueur et position du texte ne sont pas tests
// Envoi un texte et sa position d'criture sur l'cran LCD de IPL5X
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
  lg      : Byte;
  i       : integer;
begin
  try
    if not ComOpen then begin IPL5X_LCD_Write:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    if POS>19 then POS:=19;
    //***** Provisoire en attendant firmware
    If line > 2 then begin
      POS:= POS+20;                                   // Offset pour lignes 3 et 4
      if line=3 then line:=1 else line:=2 ;           // reste sur la ligne d'origine afficheur
    end;
    //***** Provisoire en attendant firmware
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_LCD;                              // l'instruction LCD
    Envoi[2] := LINE-1;                               // Ligne N 1 = 0
    Envoi[3] := POS;                                  // position X sur la ligne entre 0 et 19 ou 20  39
    lg := Length(TEXT);
    Envoi[4] := lg;                                   // nombre de caractres
    for i:=1 to lg do Envoi[4+i]:=Ord(TEXT[i]);       // pour envoyer le texte
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_LCD_Write:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_LCD_Write:=-1910;                           // erreur lors de l'excution de IPL5X_LCD_Write
    ShowMessage('Except dans IPL5X_LCD_Write.');      // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_LCD_SetCGRAM ; //(CHAR : Byte ; CO : Byte ; C1 : Byte ) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Ncaractre, C0-C7 = codage ligne
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Cration de caractre personnalis 1  8
// Description des 7 lignes successives de haut en bas en C0 -> C7
// Valeur possible 0  31, chaque case code inverse de lecture 16,8,4,2,1
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;

begin
  try
    if not ComOpen then begin IPL5X_LCD_SetCGRAM:= -1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_LCD;                              // l'instruction LCD
    Envoi[2] := LCD_CGRAM;                            // Commande criture en CGRAM
    Envoi[3] := CHAR;                                 // Caractere choisi
    Envoi[4] := C0;                                   // Ligne calcule 1
    Envoi[5] := C1;                                   // Ligne calcule 2
    Envoi[6] := C2;                                   // Ligne calcule 3
    Envoi[7] := C3;                                   // Ligne calcule 4
    Envoi[8] := C4;                                   // Ligne calcule 5
    Envoi[9] := C5;                                   // Ligne calcule 6
    Envoi[10] := C6;                                   // Ligne calcule 7
    Envoi[11] := C7;                                  // Ligne calcule 8
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_LCD_SetCGRAM:=IPL5X_Send(@saData,@Fast);    // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_LCD_SetCGRAM:=-1920;                        // erreur lors de l'excution de IPL5X_LCD_SetCGRAM
    ShowMessage('Except dans IPL5X_LCD_SetCGRAM');    // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_LCD_ChrWrite; //(LINE : Byte ; POS : Byte ; CHAR : Byte) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Ligne, Position, Caractre
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Ecriture sur LCD, en LINE,POS d'un caractre spcial redfini en CGRAM
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;

begin
  try
    if not ComOpen then begin IPL5X_LCD_ChrWrite:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_LCD;                              // l'instruction LCD
    Envoi[2] := LINE-1;                               // Ligne N 1 = 0
    Envoi[3] := POS;                                  // position X sur la ligne entre 0 et 19
    Envoi[4] := 1;                                    // Un seul caractre
    Envoi[5] := CHAR;                                 // Adresse CGRAM du caractre
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_LCD_ChrWrite:=IPL5X_Send(@saData,@Fast);    // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_LCD_ChrWrite:=-1930;                        // erreur lors de l'excution de IPL5X_LCD_Write
    ShowMessage('Except dans IPL5X_LCD_ChrWrite.');   // debug uniquement
  end;
end;


//-----------------------------------------------------------------------------
function IPL5X_GetActiveTable; //(var NBR : Byte) : Longint;
//-----------------------------------------------------------------------------
// Appel  : nant
// Retour : Numro de la table active dans NBR, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi les commandes IPL5X : Read Table Number (0x54 0x00)
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_GetActiveTable:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_TABLE;                            // l'instruction TABLE 0x54
    Envoi[2] := $00;                                  // CMD pour lire le n de table
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_GetActiveTable:=IPL5X_Send(@saData,@Fast);  // envoi pointeurs  IPL5X par la DLL et retour code erreur
    NBR      := pBuff^[3];                            // NBR renvoie le contenu du 4me octet des donnes SAFEARRAY
  except
    IPL5X_GetActiveTable:=-2100;                      // erreur lors de l'excution de IPL5X_GetActiveTable
    ShowMessage('Except dans IPL5X_GetActiveTable.'); // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_SetActiveTable; //(var NBR : Byte ; RET : Byte) : Longint;
//-----------------------------------------------------------------------------
// Appel  : NBR = Numro de la table  activer
// Retour : RET=$00 si OK RET=$FF si echec, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi les commandes IPL5X : Write Table Number (0x54 0x01 NBR)
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_SetActiveTable:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_TABLE;                            // l'instruction TABLE 0x54
    Envoi[2] := $01;                                  // CMD pour crire le n de table
    Envoi[3] := NBR;                                  // n de la table  crire en mmoire
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_SetActiveTable:=IPL5X_Send(@saData,@Fast);  // envoi pointeurs  IPL5X par la DLL et retour code erreur
    RET      := pBuff^[2];                            // RET renvoie le contenu du 2me octet des donnes SAFEARRAY
  except
    IPL5X_SetActiveTable:=-2110;                      // erreur lors de l'excution de IPL5X_SetActiveTable
    ShowMessage('Except dans IPL5X_SetActiveTable.'); // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_ReadTable; //(var NBR : Byte ; var Table : IPL5X_Table) : Longint;
//-----------------------------------------------------------------------------
// Appel  : NBR (il sera aussi renvoy dans Table)
// Retour : Contenu de structure IPL5X_Table, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi les commandes IPL5X : Read Table Low, High et Upper
// et traite le retour sous forme d'une seule structure IPL5X_Table
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
  TbLow   : IPL5X_Table_Low;
  pLow    : PIPL5X_Table_Low;
  TbHigh  : IPL5X_Table_High;
  pHigh   : PIPL5X_Table_High;
  TbUpper : IPL5X_Table_Upper;
  pUpper  : PIPL5X_Table_Upper;
  i       : integer;
begin
  try
    if not ComOpen then begin IPL5X_ReadTable:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    // Lecture TABLE LOW
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_TABLE;                            // l'instruction TABLE 0x54
    Envoi[2] := $03;                                  // CMD pour lire table low
    Envoi[3] := NBR;                                  // n de la table
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_ReadTable:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pLow     := saData.pvData;                        // pLow pointe sur les datas du safearray
    TbLow    := pLow^;                                // TbLow rcupre les donnes  l'adresse du pointeur
    // Lecture TABLE HIGH
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_TABLE;                            // l'instruction TABLE 0x54
    Envoi[2] := $05;                                  // CMD pour lire table high
    Envoi[3] := NBR;                                  // n de la table
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_ReadTable:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pHigh    := saData.pvData;                        // pHigh pointe sur les datas du safearray
    TbHigh   := pHigh^;                               // TbHigh rcupre les donnes  l'adresse du pointeur
    // Lecture TABLE UPPER
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_TABLE;                            // l'instruction TABLE 0x54
    Envoi[2] := $07;                                  // CMD pour lire table upper
    Envoi[3] := NBR;                                  // n de la table
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_ReadTable:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pUpper   := saData.pvData;                        // pUpper pointe sur les datas du safearray
    TbUpper  := pUpper^;                              // TbUpper rcupre les donnes  l'adresse du pointeur
    // Traitement pour affectation TABLE retour IPL5X_Table
    Table.NBR   := TbLow.NBR;
    for i:=1 to  8 do Table.NAME[i]  := TbLow.NAME[i];
    for i:=1 to  5 do Table.A[i]     := TbLow.A[i];
    for i:=1 to 10 do Table.OUT[i]   := TbLow.OUT[i];
    for i:=1 to  5 do Table.S[i]     := TbHigh.S[i];
    for i:=1 to  5 do Table.VMSA[i]  := TbHigh.VMSA[i];
    for i:=1 to  5 do Table.VMAA[i]  := TbHigh.VMAA[i];
    for i:=1 to  5 do Table.ORI[i]   := TbUpper.ORI[i];
    Table.FREQ  := TbLow.FREQ;
    Table.SLOPE := TbLow.SLOPE;
    Table.PWM   := TbLow.PWM;
    Table.MOFF  := TbLow.MOFF;
    Table.FLAGS := TbLow.FLAGS;
    Table.IO1   := TbLow.IO1;
    Table.LANG  := TbLow.LANG;
  except
    IPL5X_ReadTable:=-2120;                           // erreur lors de l'excution de IPL5X_ReadTable
    ShowMessage('Except dans IPL5X_ReadTable.');      // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_WriteTable; //( NBR : Byte ; Table : IPL5X_Table) : Longint;
//-----------------------------------------------------------------------------
// Appel  : NBR, Table structure IPL5X_Table(Contient aussi Table.NBR qui ne sera pas considr)
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Extrait les donnes Low, High et Upper de la structure IPL5X_Table
// Envoi les commandes IPL5X : Write Table Low, High et Upper
// si NBR et Table.NBR sont diffrent c'est NBR qui est utilis, ceci permet de dupliquer une table
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Fast    : LongInt;
  TbLow   : IPL5X_Table_Low;
  pLow    : PIPL5X_Table_Low;
  TbHigh  : IPL5X_Table_High;
  pHigh   : PIPL5X_Table_High;
  TbUpper : IPL5X_Table_Upper;
  pUpper  : PIPL5X_Table_Upper;
  i       : integer;
begin
  try
    if not ComOpen then begin IPL5X_WriteTable:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    // extraction et criture de Table Low
    TbLow.ID    := 0;                                 // ID toujours  0
    TbLow.INST  := IPL_TABLE;                         // Instruction Table=0x54
    TbLow.CMD   := $04;                               // CMD pour crire la Table Low
    TbLow.NBR   := NBR;                               // n de table envoy
    for i:=1 to  8 do TbLow.NAME[i] := Table.NAME[i];
    for i:=1 to  5 do TbLow.A[i]    := Table.A[i];
    for i:=1 to 10 do TbLow.OUT[i]  := Table.OUT[i];
    TbLow.FREQ  := Table.FREQ;
    TbLow.SLOPE := Table.SLOPE;
    TbLow.PWM   := Table.PWM;
    TbLow.MOFF  := Table.MOFF;
    TbLow.FLAGS := Table.FLAGS;
    TbLow.IO1   := Table.IO1;
    TbLow.LANG  := Table.LANG;
    pLow        := saData.pvData;                     // pLow pointe sur les datas du safearray
    pLow^       := TbLow;                             // on met le contenu de TbLow  l'adresse du safearray
    Fast        := 1;                                 // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_WriteTable:=IPL5X_Send(@saData,@Fast);      // envoi pointeurs  IPL5X par la DLL et retour code erreur
    // tester RET<>$FF
    // extraction et criture de Table High
    TbHigh.ID   := 0;                                 // ID toujours  0
    TbHigh.INST := IPL_TABLE;                         // Instruction Table=0x54
    TbHigh.CMD  := $06;                               // CMD pour crire la Table High
    TbHigh.NBR  := NBR;                               // n de table envoy
    for i:=1 to 5 do TbHigh.S[i]    := Table.S[i];
    for i:=1 to 5 do TbHigh.VMSA[i] := Table.VMSA[i];
    for i:=1 to 5 do TbHigh.VMAA[i] := Table.VMAA[i];
    pHigh       := saData.pvData;                     // pHigh pointe sur les datas du safearray
    pHigh^      := TbHigh;                            // on met le contenu de TbHigh  l'adresse du safearray
    Fast        := 1;                                 // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_WriteTable:=IPL5X_Send(@saData,@Fast);      // envoi pointeurs  IPL5X par la DLL et retour code erreur
    // tester RET<>$FF
    // extraction et criture de Table Upper
    TbUpper.ID   := 0;                                // ID toujours  0
    TbUpper.INST := IPL_TABLE;                        // Instruction Table=0x54
    TbUpper.CMD  := $08;                              // CMD pour crire la Table Upper
    TbUpper.NBR  := NBR;                              // n de table envoy
    for i:=1 to 5 do TbUpper.ORI[i]:= Table.ORI[i];
    pUpper       := saData.pvData;                    // pUpper pointe sur les datas du safearray
    pUpper^      := TbUpper;                          // on met le contenu de TbHigh  l'adresse du safearray
    Fast         := 1;                                // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_WriteTable:=IPL5X_Send(@saData,@Fast);      // envoi pointeurs  IPL5X par la DLL et retour code erreur
    // tester RET<>$FF
  except
    IPL5X_WriteTable:=-2130;                          // erreur lors de l'excution de IPL5X_ReadTable
    ShowMessage('Except dans IPL5X_WriteTable.');     // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_StatusOverride; //(var OVER:boolean, SENSOR:boolean, USE:boolean) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande $4F $02
// Retour : Etat des capteurs, fin de course et status d'utilisation , Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande Override $4F $02, analyse le retour pour renvoyer l'tat des capteurs
// et leur prise en compte ou non.
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
  RET     : Byte;
begin
  try
    if not ComOpen then begin IPL5X_StatusOverride:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_OVERRIDE;                         // l'instruction OVERRIDE 0x4F
    Envoi[2] := OVR_STATUS;                           // CMD pour lire l'tat des override
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_StatusOverride:=IPL5X_Send(@saData,@Fast);  // envoi pointeurs  IPL5X par la DLL et retour code erreur
    RET      := pBuff^[2];                            // NBR renvoie le contenu du 2me octet des donnes SAFEARRAY
    if (RET and OVR_USE)=0        then USE:=true
                                  else Use:=false;    // les overrides sont pris en compte ou non
    if (RET and OVR_TABLE)=0      then USE:=false;    // la table n'est pas configure pour utiliser les overrides
    if (RET and OVR_AXIS_LIMIT)=0 then OVER:=false
                                  else OVER:=true;    // contact fin de course ouvert ou non
    if (RET and OVR_SENSOR)=0     then SENSOR:=false
                                  else SENSOR:=true;  // contact palpeur ouvert ou non
  except
    IPL5X_StatusOverride:=-2200;                      // erreur lors de l'excution de IPL5X_StatusOverride
    ShowMessage('Except dans IPL5X_StatusOverride.'); // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_UseOverride; //(USE:boolean) : Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande $4F avec paramtre USE : USE=true==> $4F $00, false==> $4F $01
// Retour : Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande Override avec code pour prise en compte ou non des fins de course et capteurs
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------
var
  Envoi   : BufferIPL5X;
  pBuff   : PBufferIPL5X;
  Fast    : LongInt;
begin
  try
    if not ComOpen then begin IPL5X_UseOverride:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Envoi[0] := 0;                                    // toujours  0
    Envoi[1] := IPL_OVERRIDE;                         // l'instruction OVERRIDE 0x4F
    if USE then Envoi[2]:=OVR_ON else Envoi[2]:=OVR_OFF; // CMD pour prendre en compte ou non les fins de course
    pBuff    := saData.pvData;                        // pBuff pointe sur les datas du safearray
    pBuff^   := Envoi;                                // on met le contenu de Envoi  l'adresse du safearray
    Fast     := 1;                                    // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_UseOverride:=IPL5X_Send(@saData,@Fast);     // envoi pointeurs  IPL5X par la DLL et retour code erreur
  except
    IPL5X_UseOverride:=-2250;                         // erreur lors de l'excution de IPL5X_UseOverride
    ShowMessage('Except dans IPL5X_UseOverride.');    // debug uniquement
  end;
end;

//************************** Chantier THL

//-----------------------------------------------------------------------------
function IPL5X_Cde_IO1; //(var CMD:Byte, VAL:Byte ; Retour : IO1_IPL5X_PC) :Longint;
//-----------------------------------------------------------------------------
// Appel  : Commande $00-$01-$02-$03, paramtre (si CMD = $03)
// Retour : Retour : Contenu de structure IO1_IPL5X_PC, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande Set_IO1  IPL5X et traite le retour dans CMD et VAL
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

var
  pIO1    : PIO1_IPL5X_PC;
  Fast    : Longint;
begin
  try
    if not ComOpen then begin IPL5X_Cde_IO1:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Retour.ID   := 0;                                 // toujours  0
    Retour.Code := IPL_IO1;                           // la commande pour Set IO1 ($47)
    Retour.CMD  := CMD;                               // Commande CMD
    Retour.VAL  := VAL;                               // Paramtre pas toujours utilis
    pIO1        := saData.pvData;                     // pIO1 pointe sur les datas du safearray
    pIO1^       := Retour;                            // on lui affecte les donnes contenues dans Retour
    Fast        := 1;                                 // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_Cde_IO1:=IPL5X_Send(@saData,@Fast);         // envoi pointeurs  IPL5X par la DLL et retour code erreur
    pIO1        := saData.pvData;                     // pStop pointe sur les datas du safearray
    Retour      := pIO1^;                             // on rcupre les infos en retour
  except
    IPL5X_Cde_IO1:=-2100;                           // erreur lors de l'excution de IPL5X_InfoFlash
    ShowMessage('Except dans IPL5X_Cde_IO1.');      // debug uniquement
  end;
end;

//-----------------------------------------------------------------------------
function IPL5X_InfoFlash; // (var Retour : Info_Flash_IPL5X_PC): Longint ;
//-----------------------------------------------------------------------------
// Appel  : Nant
// Retour : Contenu de structure Info_Flash_IPL5X_PC, Code Erreur
//-----------------------------------------------------------------------------
// Description :
// Envoi de la commande IPL_FLASH_MODEL($46) vers IPL5X et traite le retour sous forme de structure INFORMATION
// Intgre la gestion des SAFEARRAY et des POINTEURS
//-----------------------------------------------------------------------------

var
  pInfo   : PInfo_Flash_IPL5_PC;
  Fast    : Longint;
begin
  try
    if not ComOpen then begin IPL5X_InfoFlash:=-1001; exit; end; // IPL5X_OpenCom pas lanc
    Retour.ID   := 0;                                 // toujours  0
    Retour.Code := IPL_FLASH_MODEL;                   // la commande pour "Type de Flash"
    pInfo       := saData.pvData;                     // pInfo pointe sur les datas du safearray
    pInfo^      := Retour;                            // on lui affecte les donnes contenues dans Retour
    Fast        := 1;                                 // 0 pour avoir les tests, 1 pour avoir la rapidit
    IPL5X_InfoFlash:=IPL5X_Send(@saData,@Fast);       // envoi pointeurs  IPL5X par la DLL et retour code erreur
    Retour      := pInfo^;                            // on rcupre les infos en retour
  except
    IPL5X_InfoFlash:=-2000;                           // erreur lors de l'excution de IPL5X_InfoFlash
    ShowMessage('Except dans IPL5X_InfoFlash.');      // debug uniquement
  end;
end;

{*****************************************************************************
  Reste a faire...
   * function IPL5x_FlashRd ($52,BCK,ADRH,ADRL)lecture d'un block en flash
   * RETOUR 0x52,BCK,ADRH,ADRL,D0..D31   (35 bytes)
   *----------------------------------------------------------------------------
   * BCK : 0-31 = Numro du block
   * ADRH : 0-255 Adresse de poids fort
   * ADRL : 0-255 Adresse de poids faible
   * D0..D31 : Data bytes.
   *****************************************************************************
    }
    {
   *****************************************************************************
   * function IPL5x_FlashBlockCLR ($45,BCK)l Effacement d'un block en flash
   * RETOUR 0x45,RET   (2 bytes)
   *----------------------------------------------------------------------------
   * BCK : 0-31 = Numro du block
   * RET : 0 = ERREUR, 1 = effacement pas ncessaire, 255 = effacement OK
   *****************************************************************************
    }
    {
   *****************************************************************************
   * function IPL5x_FlashWr ($57,BCK,ADRH,ADRL,...)Ecriture d'un block en flash
   * RETOUR 0x57,BCK,ADRH,ADRL,D0..D31   (35 bytes)
   *----------------------------------------------------------------------------
   * BCK : 0-31 = Numro du block
   * ADRH : 0-255 Adresse de poids fort
   * ADRL : 0-255 Adresse de poids faible
   * D0..D31 : Data bytes.
   * ne pas oublier d'incrmenter ADRL+H de 32  chaque criture !
   *----------------------------------------------------------------------------
   * BCK : 0-31 = Numro du block
   * RET : 0 = ERREUR,  255 = criture OK
   ***************************************************************************** }





   
//=============================================================================
end.
// FIN D'UNITE - IPL5Xcom =====================================================
